@@ -1,23 +1,22 @@
-/* $Id: move.c 5661 2016-02-22 12:49:08Z bens $ */
 /**************************************************************************
- *   move.c                                                               *
+ *   move.c  --  This file is part of GNU nano.                           *
  *                                                                        *
  *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
  *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
- *   This program is free software; you can redistribute it and/or modify *
- *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 3, or (at your option)  *
- *   any later version.                                                   *
+ *   Copyright (C) 2014, 2015, 2016 Benno Schulenberg                     *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful, but  *
- *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
- *   General Public License for more details.                             *
+ *   GNU nano is free software: you can redistribute it and/or modify     *
+ *   it under the terms of the GNU General Public License as published    *
+ *   by the Free Software Foundation, either version 3 of the License,    *
+ *   or (at your option) any later version.                               *
+ *                                                                        *
+ *   GNU nano is distributed in the hope that it will be useful,          *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty          *
+ *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.              *
+ *   See the GNU General Public License for more details.                 *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
- *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
- *   02110-1301, USA.                                                     *
+ *   along with this program.  If not, see http://www.gnu.org/licenses/.  *
  *                                                                        *
  **************************************************************************/
 
@@ -29,11 +28,11 @@
 /* Move to the first line of the file. */
 void do_first_line(void)
 {
-    openfile->current = openfile->edittop = openfile->fileage;
+    openfile->current = openfile->fileage;
     openfile->current_x = 0;
     openfile->placewewant = 0;
 
-    edit_refresh_needed = TRUE;
+    refresh_needed = TRUE;
 }
 
 /* Move to the last line of the file. */
@@ -42,23 +41,26 @@ void do_last_line(void)
     openfile->current = openfile->filebot;
     openfile->current_x = strlen(openfile->filebot->data);
     openfile->placewewant = xplustabs();
+
+    /* Set the last line of the screen as the target for the cursor. */
     openfile->current_y = editwinrows - 1;
 
-    edit_refresh_needed = TRUE;
+    refresh_needed = TRUE;
+    focusing = FALSE;
 }
 
 /* Move up one page. */
 void do_page_up(void)
 {
-    int i, skipped = 0;
+    int i, mustmove, skipped = 0;
 
     /* If the cursor is less than a page away from the top of the file,
      * put it at the beginning of the first line. */
     if (openfile->current->lineno == 1 || (
 #ifndef NANO_TINY
-	!ISSET(SOFTWRAP) &&
+		!ISSET(SOFTWRAP) &&
 #endif
-	openfile->current->lineno <= editwinrows - 2)) {
+		openfile->current->lineno <= editwinrows - 2)) {
 	do_first_line();
 	return;
     }
@@ -66,16 +68,16 @@ void do_page_up(void)
     /* If we're not in smooth scrolling mode, put the cursor at the
      * beginning of the top line of the edit window, as Pico does. */
 #ifndef NANO_TINY
-    if (!ISSET(SMOOTH_SCROLL)) {
+    if (!ISSET(SMOOTH_SCROLL))
 #endif
+    {
 	openfile->current = openfile->edittop;
 	openfile->placewewant = openfile->current_y = 0;
-#ifndef NANO_TINY
     }
-#endif
 
-    for (i = editwinrows - 2; i - skipped > 0 && openfile->current !=
-	openfile->fileage; i--) {
+    mustmove = (editwinrows < 3) ? 1 : editwinrows - 2;
+
+    for (i = mustmove; i - skipped > 0 && openfile->current != openfile->fileage; i--) {
 	openfile->current = openfile->current->prev;
 #ifndef NANO_TINY
 	if (ISSET(SOFTWRAP) && openfile->current) {
@@ -89,26 +91,26 @@ void do_page_up(void)
     }
 
     openfile->current_x = actual_x(openfile->current->data,
-	openfile->placewewant);
+					openfile->placewewant);
 
 #ifdef DEBUG
     fprintf(stderr, "do_page_up: openfile->current->lineno = %lu, skipped = %d\n",
-	(unsigned long)openfile->current->lineno, skipped);
+			(unsigned long)openfile->current->lineno, skipped);
 #endif
 
     /* Scroll the edit window up a page. */
-    edit_update(NONE);
+    adjust_viewport(STATIONARY);
+    refresh_needed = TRUE;
 }
 
 /* Move down one page. */
 void do_page_down(void)
 {
-    int i;
+    int i, mustmove;
 
     /* If the cursor is less than a page away from the bottom of the file,
      * put it at the end of the last line. */
-    if (openfile->current->lineno + maxrows - 2 >=
-	openfile->filebot->lineno) {
+    if (openfile->current->lineno + maxrows - 2 >= openfile->filebot->lineno) {
 	do_last_line();
 	return;
     }
@@ -116,16 +118,16 @@ void do_page_down(void)
     /* If we're not in smooth scrolling mode, put the cursor at the
      * beginning of the top line of the edit window, as Pico does. */
 #ifndef NANO_TINY
-    if (!ISSET(SMOOTH_SCROLL)) {
+    if (!ISSET(SMOOTH_SCROLL))
 #endif
+    {
 	openfile->current = openfile->edittop;
 	openfile->placewewant = openfile->current_y = 0;
-#ifndef NANO_TINY
     }
-#endif
 
-    for (i = maxrows - 2; i > 0 && openfile->current !=
-	openfile->filebot; i--) {
+    mustmove = (maxrows < 3) ? 1 : maxrows - 2;
+
+    for (i = mustmove; i > 0 && openfile->current != openfile->filebot; i--) {
 	openfile->current = openfile->current->next;
 #ifdef DEBUG
 	fprintf(stderr, "do_page_down: moving to line %lu\n", (unsigned long)openfile->current->lineno);
@@ -134,10 +136,11 @@ void do_page_down(void)
     }
 
     openfile->current_x = actual_x(openfile->current->data,
-	openfile->placewewant);
+					openfile->placewewant);
 
     /* Scroll the edit window down a page. */
-    edit_update(NONE);
+    adjust_viewport(STATIONARY);
+    refresh_needed = TRUE;
 }
 
 #ifndef DISABLE_JUSTIFY
@@ -146,8 +149,7 @@ void do_page_down(void)
  * afterwards. */
 void do_para_begin(bool allow_update)
 {
-    filestruct *current_save = openfile->current;
-    const size_t pww_save = openfile->placewewant;
+    filestruct *was_current = openfile->current;
 
     if (openfile->current != openfile->fileage) {
 	do {
@@ -157,10 +159,9 @@ void do_para_begin(bool allow_update)
     }
 
     openfile->current_x = 0;
-    openfile->placewewant = 0;
 
     if (allow_update)
-	edit_redraw(current_save, pww_save);
+	edit_redraw(was_current);
 }
 
 /* Move up to the beginning of the last beginning-of-paragraph line
@@ -178,16 +179,15 @@ void do_para_begin_void(void)
  * paragraph or isn't in a paragraph. */
 void do_para_end(bool allow_update)
 {
-    filestruct *const current_save = openfile->current;
-    const size_t pww_save = openfile->placewewant;
+    filestruct *was_current = openfile->current;
 
     while (openfile->current != openfile->filebot &&
-	!inpar(openfile->current))
+		!inpar(openfile->current))
 	openfile->current = openfile->current->next;
 
     while (openfile->current != openfile->filebot &&
-	inpar(openfile->current->next) &&
-	!begpar(openfile->current->next)) {
+		inpar(openfile->current->next) &&
+		!begpar(openfile->current->next)) {
 	openfile->current = openfile->current->next;
 	openfile->current_y++;
     }
@@ -195,14 +195,11 @@ void do_para_end(bool allow_update)
     if (openfile->current != openfile->filebot) {
 	openfile->current = openfile->current->next;
 	openfile->current_x = 0;
-	openfile->placewewant = 0;
-    } else {
+    } else
 	openfile->current_x = strlen(openfile->current->data);
-	openfile->placewewant = xplustabs();
-    }
 
     if (allow_update)
-	edit_redraw(current_save, pww_save);
+	edit_redraw(was_current);
 }
 
 /* Move down to the beginning of the last line of the current paragraph.
@@ -214,14 +211,52 @@ void do_para_end_void(void)
 }
 #endif /* !DISABLE_JUSTIFY */
 
-#ifndef NANO_TINY
+/* Move to the preceding block of text in the file. */
+void do_prev_block(void)
+{
+    filestruct *was_current = openfile->current;
+    bool is_text = FALSE, seen_text = FALSE;
+
+    /* Skip backward until first blank line after some nonblank line(s). */
+    while (openfile->current->prev != NULL && (!seen_text || is_text)) {
+	openfile->current = openfile->current->prev;
+	is_text = !white_string(openfile->current->data);
+	seen_text = seen_text || is_text;
+    }
+
+    /* Step forward one line again if this one is blank. */
+    if (openfile->current->next != NULL &&
+		white_string(openfile->current->data))
+	openfile->current = openfile->current->next;
+
+    openfile->current_x = 0;
+    edit_redraw(was_current);
+}
+
+/* Move to the next block of text in the file. */
+void do_next_block(void)
+{
+    filestruct *was_current = openfile->current;
+    bool is_white = white_string(openfile->current->data);
+    bool seen_white = is_white;
+
+    /* Skip forward until first nonblank line after some blank line(s). */
+    while (openfile->current->next != NULL && (!seen_white || is_white)) {
+	openfile->current = openfile->current->next;
+	is_white = white_string(openfile->current->data);
+	seen_white = seen_white || is_white;
+    }
+
+    openfile->current_x = 0;
+    edit_redraw(was_current);
+}
+
 /* Move to the previous word in the file.  If allow_punct is TRUE, treat
  * punctuation as part of a word.  If allow_update is TRUE, update the
  * screen afterwards. */
 void do_prev_word(bool allow_punct, bool allow_update)
 {
-    size_t pww_save = openfile->placewewant;
-    filestruct *current_save = openfile->current;
+    filestruct *was_current = openfile->current;
     bool seen_a_word = FALSE, step_forward = FALSE;
 
     assert(openfile->current != NULL && openfile->current->data != NULL);
@@ -257,11 +292,12 @@ void do_prev_word(bool allow_punct, bool allow_update)
 	/* Move one character forward again to sit on the start of the word. */
 	openfile->current_x = move_mbright(openfile->current->data,
 						openfile->current_x);
-    openfile->placewewant = xplustabs();
 
     /* If allow_update is TRUE, update the screen. */
-    if (allow_update)
-	edit_redraw(current_save, pww_save);
+    if (allow_update) {
+	focusing = FALSE;
+	edit_redraw(was_current);
+    }
 }
 
 /* Move to the previous word in the file, treating punctuation as part of a
@@ -277,8 +313,7 @@ void do_prev_word_void(void)
  * otherwise. */
 bool do_next_word(bool allow_punct, bool allow_update)
 {
-    size_t pww_save = openfile->placewewant;
-    filestruct *current_save = openfile->current;
+    filestruct *was_current = openfile->current;
     bool started_on_word = is_word_mbchar(openfile->current->data +
 				openfile->current_x, allow_punct);
     bool seen_space = !started_on_word;
@@ -310,11 +345,11 @@ bool do_next_word(bool allow_punct, bool allow_update)
 	    break;
     }
 
-    openfile->placewewant = xplustabs();
-
     /* If allow_update is TRUE, update the screen. */
-    if (allow_update)
-	edit_redraw(current_save, pww_save);
+    if (allow_update) {
+	focusing = FALSE;
+	edit_redraw(was_current);
+    }
 
     /* Return whether we started on a word. */
     return started_on_word;
@@ -326,7 +361,19 @@ void do_next_word_void(void)
 {
     do_next_word(ISSET(WORD_BOUNDS), TRUE);
 }
-#endif /* !NANO_TINY */
+
+/* Make sure that the current line, when it is partially scrolled off the
+ * screen in softwrap mode, is scrolled fully into view. */
+void ensure_line_is_visible(void)
+{
+#ifndef NANO_TINY
+    if (ISSET(SOFTWRAP) && strlenpt(openfile->current->data) / editwincols +
+				openfile->current_y >= editwinrows) {
+	adjust_viewport(ISSET(SMOOTH_SCROLL) ? FLOWING : CENTERING);
+	refresh_needed = TRUE;
+    }
+#endif
+}
 
 /* Move to the beginning of the current line.  If the SMART_HOME flag is
  * set, move to the first non-whitespace character of the current line
@@ -334,7 +381,7 @@ void do_next_word_void(void)
  * if we are. */
 void do_home(void)
 {
-    size_t pww_save = openfile->placewewant;
+    size_t was_column = xplustabs();
 
 #ifndef NANO_TINY
     if (ISSET(SMART_HOME)) {
@@ -345,42 +392,36 @@ void do_home(void)
 	if (openfile->current_x == current_x_save ||
 		openfile->current->data[openfile->current_x] == '\0')
 	    openfile->current_x = 0;
-
-	openfile->placewewant = xplustabs();
-    } else {
+    } else
 #endif
 	openfile->current_x = 0;
-	openfile->placewewant = 0;
-#ifndef NANO_TINY
-    }
-#endif
 
-    if (need_screen_update(pww_save))
+    openfile->placewewant = xplustabs();
+
+    if (need_horizontal_scroll(was_column, openfile->placewewant))
 	update_line(openfile->current, openfile->current_x);
 }
 
 /* Move to the end of the current line. */
 void do_end(void)
 {
-    size_t pww_save = openfile->placewewant;
+    size_t was_column = xplustabs();
 
     openfile->current_x = strlen(openfile->current->data);
     openfile->placewewant = xplustabs();
 
-    if (need_screen_update(pww_save))
+    if (need_horizontal_scroll(was_column, openfile->placewewant))
 	update_line(openfile->current, openfile->current_x);
+
+    ensure_line_is_visible();
 }
 
 /* If scroll_only is FALSE, move up one line.  If scroll_only is TRUE,
  * scroll up one line without scrolling the cursor. */
-void do_up(
-#ifndef NANO_TINY
-	bool scroll_only
-#else
-	void
-#endif
-	)
+void do_up(bool scroll_only)
 {
+    size_t was_column = xplustabs();
+
     /* If we're at the top of the file, or if scroll_only is TRUE and
      * the top of the file is onscreen, get out. */
     if (openfile->current == openfile->fileage
@@ -395,67 +436,41 @@ void do_up(
     /* Move the current line of the edit window up. */
     openfile->current = openfile->current->prev;
     openfile->current_x = actual_x(openfile->current->data,
-	openfile->placewewant);
+					openfile->placewewant);
 
-    /* If scroll_only is FALSE and if we're on the first line of the
-     * edit window, scroll the edit window up one line if we're in
-     * smooth scrolling mode, or up half a page if we're not.  If
-     * scroll_only is TRUE, scroll the edit window up one line
-     * unconditionally. */
-    if (openfile->current_y == 0
-#ifndef NANO_TINY
-	|| (ISSET(SOFTWRAP) && openfile->edittop->lineno == openfile->current->next->lineno) || scroll_only
-#endif
-	)
-	edit_scroll(UPWARD,
-#ifndef NANO_TINY
-		(ISSET(SMOOTH_SCROLL) || scroll_only) ? 1 :
-#endif
-		editwinrows / 2 + 1);
+    /* When the cursor was on the first line of the edit window (or when just
+     * scrolling without moving the cursor), scroll the edit window up -- one
+     * line if we're in smooth scrolling mode, and half a page otherwise. */
+    if (openfile->current->next == openfile->edittop || scroll_only)
+	edit_scroll(UPWARD, (ISSET(SMOOTH_SCROLL) || scroll_only) ?
+				1 : editwinrows / 2 + 1);
 
-    /* If we're below the first line of the edit window, update the
-     * line we were on before and the line we're on now.  The former
-     * needs to be redrawn if we're not on the first page, and the
-     * latter needs to be drawn unconditionally. */
+    /* If the lines weren't already redrawn, see if they need to be. */
     if (openfile->current_y > 0) {
-	if (need_screen_update(0))
+	/* Redraw the prior line if it was horizontally scrolled. */
+	if (need_horizontal_scroll(was_column, 0))
 	    update_line(openfile->current->next, 0);
-	update_line(openfile->current, openfile->current_x);
+	/* Redraw the current line if it needs to be horizontally scrolled. */
+	if (need_horizontal_scroll(0, xplustabs()))
+	    update_line(openfile->current, openfile->current_x);
     }
 }
 
 /* Move up one line. */
 void do_up_void(void)
 {
-    do_up(
-#ifndef NANO_TINY
-	FALSE
-#endif
-	);
+    do_up(FALSE);
 }
 
-#ifndef NANO_TINY
-/* Scroll up one line without scrolling the cursor. */
-void do_scroll_up(void)
-{
-    do_up(TRUE);
-}
-#endif
-
 /* If scroll_only is FALSE, move down one line.  If scroll_only is TRUE,
  * scroll down one line without scrolling the cursor. */
-void do_down(
-#ifndef NANO_TINY
-	bool scroll_only
-#else
-	void
-#endif
-	)
+void do_down(bool scroll_only)
 {
 #ifndef NANO_TINY
     int amount = 0, enough;
     filestruct *topline;
 #endif
+    size_t was_column = xplustabs();
 
     /* If we're at the bottom of the file, get out. */
     if (openfile->current == openfile->filebot)
@@ -465,76 +480,79 @@ void do_down(
 		openfile->current->lineno - openfile->edittop->lineno);
     assert(openfile->current->next != NULL);
 
-    /* Move the current line of the edit window down. */
-    openfile->current = openfile->current->next;
-    openfile->current_x = actual_x(openfile->current->data,
-	openfile->placewewant);
-
 #ifndef NANO_TINY
     if (ISSET(SOFTWRAP)) {
-	/* Compute the amount to scroll. */
-	amount = (strlenpt(openfile->current->data) / COLS + openfile->current_y + 2
-		 + strlenpt(openfile->current->prev->data) / COLS - editwinrows);
+	/* Compute the number of lines to scroll. */
+	amount = strlenpt(openfile->current->data) / editwincols -
+			xplustabs() / editwincols +
+			strlenpt(openfile->current->next->data) / editwincols +
+			openfile->current_y - editwinrows + 2;
 	topline = openfile->edittop;
 	/* Reduce the amount when there are overlong lines at the top. */
 	for (enough = 1; enough < amount; enough++) {
-	    amount -= strlenpt(topline->data) / COLS;
-	    if (amount <= 0) {
+	    amount -= strlenpt(topline->data) / editwincols;
+	    if (amount > 0)
+		topline = topline->next;
+	    if (amount < enough) {
 		amount = enough;
 		break;
 	    }
-	    topline = topline->next;
 	}
     }
 #endif
 
+    /* Move to the next line in the file. */
+    openfile->current = openfile->current->next;
+    openfile->current_x = actual_x(openfile->current->data,
+					openfile->placewewant);
+
     /* If scroll_only is FALSE and if we're on the last line of the
      * edit window, scroll the edit window down one line if we're in
      * smooth scrolling mode, or down half a page if we're not.  If
      * scroll_only is TRUE, scroll the edit window down one line
      * unconditionally. */
-    if (openfile->current_y == editwinrows - 1
-#ifndef NANO_TINY
-	|| amount > 0 || scroll_only
-#endif
-	) {
 #ifndef NANO_TINY
+    if (openfile->current_y == editwinrows - 1 || amount > 0 || scroll_only) {
 	if (amount < 1 || scroll_only)
 	    amount = 1;
-#endif
-	edit_scroll(DOWNWARD,
-#ifndef NANO_TINY
-		(ISSET(SMOOTH_SCROLL) || scroll_only) ? amount :
-#endif
-		editwinrows / 2 + 1);
-	edit_refresh_needed = TRUE;
+
+	edit_scroll(DOWNWARD, (ISSET(SMOOTH_SCROLL) || scroll_only) ?
+				amount : editwinrows / 2 + 1);
+
+	if (ISSET(SOFTWRAP)) {
+	    refresh_needed = TRUE;
+	    return;
+	}
     }
-    /* If we're above the last line of the edit window, update the line
-     * we were on before and the line we're on now.  The former needs to
-     * be redrawn if we're not on the first page, and the latter needs
-     * to be drawn unconditionally. */
-    if (openfile->current_y < editwinrows - 1
-#ifndef NANO_TINY
-	|| ISSET(SOFTWRAP)
+#else
+    if (openfile->current_y == editwinrows - 1)
+	edit_scroll(DOWNWARD, editwinrows / 2 + 1);
 #endif
-	) {
-	if (need_screen_update(0))
+
+    /* If the lines weren't already redrawn, see if they need to be. */
+    if (openfile->current_y < editwinrows - 1) {
+	/* Redraw the prior line if it was horizontally scrolled. */
+	if (need_horizontal_scroll(was_column, 0))
 	    update_line(openfile->current->prev, 0);
-	update_line(openfile->current, openfile->current_x);
+	/* Redraw the current line if it needs to be horizontally scrolled. */
+	if (need_horizontal_scroll(0, xplustabs()))
+	    update_line(openfile->current, openfile->current_x);
     }
 }
 
 /* Move down one line. */
 void do_down_void(void)
 {
-    do_down(
-#ifndef NANO_TINY
-	FALSE
-#endif
-	);
+    do_down(FALSE);
 }
 
 #ifndef NANO_TINY
+/* Scroll up one line without scrolling the cursor. */
+void do_scroll_up(void)
+{
+    do_up(TRUE);
+}
+
 /* Scroll down one line without scrolling the cursor. */
 void do_scroll_down(void)
 {
@@ -545,11 +563,11 @@ void do_scroll_down(void)
 /* Move left one character. */
 void do_left(void)
 {
-    size_t pww_save = openfile->placewewant;
+    size_t was_column = xplustabs();
 
     if (openfile->current_x > 0)
 	openfile->current_x = move_mbleft(openfile->current->data,
-		openfile->current_x);
+						openfile->current_x);
     else if (openfile->current != openfile->fileage) {
 	do_up_void();
 	openfile->current_x = strlen(openfile->current->data);
@@ -557,27 +575,35 @@ void do_left(void)
 
     openfile->placewewant = xplustabs();
 
-    if (need_screen_update(pww_save))
+    if (need_horizontal_scroll(was_column, openfile->placewewant))
 	update_line(openfile->current, openfile->current_x);
 }
 
 /* Move right one character. */
 void do_right(void)
 {
-    size_t pww_save = openfile->placewewant;
+    size_t was_column = xplustabs();
 
     assert(openfile->current_x <= strlen(openfile->current->data));
 
     if (openfile->current->data[openfile->current_x] != '\0')
 	openfile->current_x = move_mbright(openfile->current->data,
-		openfile->current_x);
+						openfile->current_x);
     else if (openfile->current != openfile->filebot) {
-	do_down_void();
 	openfile->current_x = 0;
+#ifndef NANO_TINY
+	if (ISSET(SOFTWRAP))
+	    openfile->current_y -= strlenpt(openfile->current->data) / COLS;
+#endif
     }
 
     openfile->placewewant = xplustabs();
 
-    if (need_screen_update(pww_save))
+    if (need_horizontal_scroll(was_column, openfile->placewewant))
 	update_line(openfile->current, openfile->current_x);
+
+    if (openfile->current_x == 0)
+	do_down_void();
+    else
+	ensure_line_is_visible();
 }
